@@ -21,6 +21,7 @@ import net.bytebuddy.build.HashCodeAndEqualsPlugin;
 import net.bytebuddy.description.type.TypeDescription;
 import net.bytebuddy.dynamic.ClassFileLocator;
 import net.bytebuddy.utility.OpenedClassReader;
+import net.bytebuddy.utility.nullability.MaybeNull;
 import org.objectweb.asm.Opcodes;
 
 import java.io.IOException;
@@ -32,7 +33,6 @@ import java.security.PrivilegedAction;
  * A wrapper object for representing a validated class file version in the format that is specified by the
  * <a href="http://docs.oracle.com/javase/specs/jvms/se7/html/jvms-4.html">JVMS</a>.
  */
-@HashCodeAndEqualsPlugin.Enhance
 public class ClassFileVersion implements Comparable<ClassFileVersion>, Serializable {
 
     /**
@@ -145,6 +145,21 @@ public class ClassFileVersion implements Comparable<ClassFileVersion>, Serializa
      */
     public static final ClassFileVersion JAVA_V20 = new ClassFileVersion(Opcodes.V20);
 
+    /**
+     * The class file version of Java 21.
+     */
+    public static final ClassFileVersion JAVA_V21 = new ClassFileVersion(Opcodes.V21);
+
+    /**
+     * The class file version of Java 22.
+     */
+    public static final ClassFileVersion JAVA_V22 = new ClassFileVersion(Opcodes.V22);
+
+    /**
+     * The class file version of Java 23.
+     */
+    public static final ClassFileVersion JAVA_V23 = new ClassFileVersion(Opcodes.V22 + 1);
+
     /**
      * A version locator for the executing JVM.
      */
@@ -184,7 +199,7 @@ public class ClassFileVersion implements Comparable<ClassFileVersion>, Serializa
      */
     public static ClassFileVersion ofMinorMajor(int versionNumber) {
         ClassFileVersion classFileVersion = new ClassFileVersion(versionNumber);
-        if (classFileVersion.getMajorVersion() <= BASE_VERSION) {
+        if (classFileVersion.getMajorVersion() > 0 && classFileVersion.getMajorVersion() <= BASE_VERSION) {
             throw new IllegalArgumentException("Class version " + versionNumber + " is not valid");
         }
         return classFileVersion;
@@ -237,6 +252,12 @@ public class ClassFileVersion implements Comparable<ClassFileVersion>, Serializa
             return JAVA_V19;
         } else if (javaVersionString.equals("1.20") || javaVersionString.equals("20")) {
             return JAVA_V20;
+        } else if (javaVersionString.equals("1.21") || javaVersionString.equals("21")) {
+            return JAVA_V21;
+        } else if (javaVersionString.equals("1.22") || javaVersionString.equals("22")) {
+            return JAVA_V22;
+        } else if (javaVersionString.equals("1.23") || javaVersionString.equals("23")) {
+            return JAVA_V23;
         } else {
             if (OpenedClassReader.EXPERIMENTAL) {
                 try {
@@ -302,6 +323,12 @@ public class ClassFileVersion implements Comparable<ClassFileVersion>, Serializa
                 return JAVA_V19;
             case 20:
                 return JAVA_V20;
+            case 21:
+                return JAVA_V21;
+            case 22:
+                return JAVA_V22;
+            case 23:
+                return JAVA_V23;
             default:
                 if (OpenedClassReader.EXPERIMENTAL && javaVersion > 0) {
                     return new ClassFileVersion(BASE_VERSION + javaVersion);
@@ -317,7 +344,7 @@ public class ClassFileVersion implements Comparable<ClassFileVersion>, Serializa
      * @return The latest officially supported Java version.
      */
     public static ClassFileVersion latest() {
-        return ClassFileVersion.JAVA_V20;
+        return ClassFileVersion.JAVA_V22;
     }
 
     /**
@@ -393,7 +420,10 @@ public class ClassFileVersion implements Comparable<ClassFileVersion>, Serializa
         if (binaryRepresentation.length < 7) {
             throw new IllegalArgumentException("Supplied byte array is too short to be a class file with " + binaryRepresentation.length + " byte");
         }
-        return ofMinorMajor(binaryRepresentation[6] << 8 | binaryRepresentation[7] & 0xFF);
+        return ofMinorMajor(binaryRepresentation[4] << 24
+                | binaryRepresentation[5] << 16
+                | binaryRepresentation[6] << 8
+                | binaryRepresentation[7]);
     }
 
     /**
@@ -411,7 +441,7 @@ public class ClassFileVersion implements Comparable<ClassFileVersion>, Serializa
      * @return The major version this instance represents.
      */
     public short getMajorVersion() {
-        return (short) (versionNumber & 0xFF);
+        return (short) (versionNumber & 0xFFFF);
     }
 
     /**
@@ -420,7 +450,7 @@ public class ClassFileVersion implements Comparable<ClassFileVersion>, Serializa
      * @return The minor version this instance represents.
      */
     public short getMinorVersion() {
-        return (short) (versionNumber >> 16);
+        return (short) (versionNumber >>> 16);
     }
 
     /**
@@ -499,6 +529,21 @@ public class ClassFileVersion implements Comparable<ClassFileVersion>, Serializa
                 : getMajorVersion() - other.getMajorVersion());
     }
 
+    @Override
+    public int hashCode() {
+        return versionNumber;
+    }
+
+    @Override
+    public boolean equals(@MaybeNull Object other) {
+        if (this == other) {
+            return true;
+        } else if (other == null || getClass() != other.getClass()) {
+            return false;
+        }
+        return versionNumber == ((ClassFileVersion) other).versionNumber;
+    }
+
     @Override
     public String toString() {
         return "Java " + getJavaVersion() + " (" + getMinorMajorVersion() + ")";
