@@ -20,9 +20,9 @@ import net.bytebuddy.build.CachedReturnPlugin;
 import net.bytebuddy.description.ByteCodeElement;
 import net.bytebuddy.description.DeclaredByType;
 import net.bytebuddy.description.ModifierReviewable;
-import net.bytebuddy.description.NamedElement;
 import net.bytebuddy.description.annotation.AnnotationDescription;
 import net.bytebuddy.description.annotation.AnnotationList;
+import net.bytebuddy.description.type.TypeDefinition;
 import net.bytebuddy.description.type.TypeDescription;
 import net.bytebuddy.matcher.ElementMatcher;
 import net.bytebuddy.utility.nullability.AlwaysNull;
@@ -41,10 +41,9 @@ import java.util.List;
  * Implementations of this interface describe a Java field. Implementations of this interface must provide meaningful
  * {@code equal(Object)} and {@code hashCode()} implementations.
  */
-public interface FieldDescription extends ByteCodeElement,
+public interface FieldDescription extends ModifierReviewable.ForFieldDescription,
         DeclaredByType.WithMandatoryDeclaration,
-        ModifierReviewable.ForFieldDescription,
-        NamedElement.WithGenericName,
+        ByteCodeElement.Member,
         ByteCodeElement.TypeDependant<FieldDescription.InDefinedShape, FieldDescription.Token> {
 
     /**
@@ -53,6 +52,12 @@ public interface FieldDescription extends ByteCodeElement,
     @AlwaysNull
     Object NO_DEFAULT_VALUE = null;
 
+    /**
+     * {@inheritDoc}
+     */
+    @Nonnull
+    TypeDefinition getDeclaringType();
+
     /**
      * Returns the type of the described field.
      *
